"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.config = exports.Config = void 0;
const utils_1 = require("./utils");
const configs_1 = require("./configs");
/**
 * Config class
 */
class Config {
    constructor() {
        if (utils_1.Utils.hasKeys(configs_1.PROVIDER_CONFIGS) && utils_1.Utils.hasKeys(configs_1.LOCAL_CONFIGS)) {
            this.providers = configs_1.PROVIDER_CONFIGS;
            this.localProvider = configs_1.LOCAL_CONFIGS;
        }
        else {
            this.providers = {};
            this.localProvider = {};
            const data = process.env;
            Object.keys(data).forEach((key) => {
                var _a;
                if (Object.prototype.hasOwnProperty.call(data, key)) {
                    const value = data[key];
                    const providerItem = (/PROVIDER_(.*)?_(.*)?/g).exec(key);
                    if (providerItem) {
                        const provider = providerItem[1].toLowerCase();
                        const type = providerItem[2].toLowerCase();
                        (_a = this.providers)[provider] || (_a[provider] = { provider: provider });
                        this.assignProvider(provider, type, value);
                    }
                    else {
                        this.assignLocals(key, value);
                    }
                }
            });
        }
    }
    assignProvider(provider, type, value) {
        switch (type) {
            case 'id':
                this.providers[provider].id = value;
                break;
            case 'secret':
                this.providers[provider].secret = value;
                break;
            case 'host':
                this.providers[provider].host = value;
                break;
            case 'stage':
                this.providers[provider].stage = value;
                break;
        }
    }
    assignLocals(key, value) {
        switch (key) {
            case 'REDIRECT_URI':
                this.localProvider.redirect_uri = value;
                break;
            case 'REDIRECT_CLIENT_URI':
                this.localProvider.redirect_client_uri = value;
                break;
            case 'TOKEN_SECRET':
                this.localProvider.token_secret = value;
                break;
        }
    }
    getConfig(provider, redirect_uri) {
        var _a;
        let result = {};
        redirect_uri || (redirect_uri = this.localProvider.redirect_uri);
        if (provider) {
            const configProvider = provider.replace(/-/g, '_');
            const stage = (_a = this.providers[configProvider]) === null || _a === void 0 ? void 0 : _a.stage;
            result = Object.assign({
                redirect_uri: utils_1.Utils.redirectUrlBuilder(redirect_uri || '', { provider: provider, stage: stage || 'dev' }),
                redirect_client_uri: utils_1.Utils.redirectUrlBuilder(this.localProvider.redirect_client_uri || '', { provider: provider }),
                provider: provider
            }, this.providers[configProvider]);
        }
        result.token_secret = this.localProvider.token_secret;
        return result;
    }
}
exports.Config = Config;
/**
 * @param provider {string} oauth provider name e.g. facebook or google
 */
function config(options) {
    let customUrl;
    if (options.host && options.stage) {
        customUrl = `https://${options.host}/${options.stage}/authentication/callback/{provider}`;
    }
    if (!process.env.REDIRECT_URI && customUrl !== undefined) {
        process.env.REDIRECT_URI = customUrl;
    }
    return (new Config()).getConfig(options.provider, customUrl);
}
exports.config = config;
