"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Provider = void 0;
const axios_1 = __importDefault(require("axios"));
const utils_1 = require("./utils");
const _1 = require(".");
const Providable_1 = require("./abstract/Providable");
/**
 * Default provider
 */
class Provider extends Providable_1.Provideable {
    constructor(config) {
        super();
        this.config = config;
    }
    signin(config) {
        const client_id = this.config.id;
        const { redirect_uri, signin_uri } = Object.assign(Object.assign({}, this.config), config);
        _1.utils.throwUndefined({ client_id, redirect_uri, signin_uri }, 'Provider - signin()');
        let params = {
            client_id: client_id || '',
            redirect_uri: redirect_uri || '',
            response_type: config.response_type || '',
            scope: config.scope || '',
            access_type: config.access_type || '',
            prompt: config.prompt || '',
            state: config.state || ''
        };
        const url = utils_1.Utils.urlBuilder(signin_uri || '', params);
        return { url };
    }
    ;
    async callback(event, options, additionalParams) {
        const { authorization, profile } = additionalParams;
        const { id, redirect_uri, secret, provider } = this.config;
        const attemptAuthorize = async () => {
            const mandatoryParams = {
                client_id: id,
                redirect_uri: redirect_uri,
                client_secret: secret,
                code: event.code
            };
            const payload = Object.assign(Object.assign({}, mandatoryParams), authorization);
            if (options.authorizationMethod === 'GET') {
                const url = utils_1.Utils.urlBuilder(options.authorizationURL || '', payload);
                try {
                    console.log(url);
                    const accessData = await axios_1.default.get(url);
                    return accessData;
                }
                catch (error) {
                    throw new Error(`attemptAuthorize error GET ${error}`);
                }
            }
            else {
                try {
                    const accessData = await axios_1.default.post(options.authorizationURL || '', { form: payload });
                    return accessData;
                }
                catch (error) {
                    throw new Error(`attemptAuthorize error POST ${error}`);
                }
            }
        };
        const createMappedProfile = async (accessData) => {
            if (!accessData) {
                throw new Error('No access data');
            }
            const { access_token, refresh_token } = accessData === null || accessData === void 0 ? void 0 : accessData.data;
            const profileToken = Object.assign(Object.assign({}, profile), { access_token });
            const url = utils_1.Utils.urlBuilder(options.profileURL || '', profileToken);
            try {
                const profileData = await axios_1.default.get(url);
                if (!profileData || !profileData.data) {
                    throw new Error('No profile data was returned.');
                }
                const profileJson = Object.assign(Object.assign({}, profileData.data), { provider: provider, at_hash: access_token, offline_access: refresh_token || '' });
                const mappedProfile = (options === null || options === void 0 ? void 0 : options.profileMap) ? options.profileMap(profileJson)
                    : profileJson;
                return mappedProfile;
            }
            catch (error) {
                throw new Error(`createMappedProfile error ${error}`);
            }
        };
        const accessData = await attemptAuthorize();
        const mappedProfile = await createMappedProfile(accessData);
        const state = event.state;
        return Object.assign({ state: state }, mappedProfile);
    }
}
exports.Provider = Provider;
