"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Utils = void 0;
const jwt = __importStar(require("jsonwebtoken"));
const uncamel_1 = require("./uncamel");
/**
 * Utilities for Serverless Authentication
 */
class Utils {
    /**
     * Creates redirectUrl
     * @param url {string} url base
     * @param provider {string} provider e.g. facebook
     */
    static redirectUrlBuilder(url, replacers) {
        url = url.replace(/{provider}/g, replacers.provider);
        if (replacers.stage) {
            url = url.replace(/{stage}/g, replacers.stage);
        }
        return url;
    }
    /**
     * Creates url with params
     * @param url {string} url base
     * @param params {object} url params
     */
    static urlBuilder(url, params) {
        return `${url}?${this.urlParams(params)}`;
    }
    /**
     * Creates &amp; separated params string
     * @param params {object}
     */
    static urlParams(params) {
        const result = [];
        for (const [key, value] of Object.entries(params)) {
            if (value)
                result.push(`${encodeURIComponent(uncamel_1.uncamel(key))}=${encodeURIComponent(value)}`);
        }
        return result.join('&');
    }
    /**
     * Creates Json Web Token with data
     * @param data {object}
     * @param config {object} with token_secret --> change to secret
     */
    static async createToken(data, secret, options) {
        return new Promise((resolve, reject) => {
            const callback = (err, encoded) => {
                if (err)
                    return reject(err);
                else
                    return resolve(encoded);
            };
            if (options)
                jwt.sign(data, secret, options, callback);
            else
                jwt.sign(data, secret, callback);
        });
    }
    /**
     * Reads Json Web Token and returns object
     * @param token {string}
     * @param config {object} with token_secret --> change to secret
     */
    static async readToken(token, secret, options) {
        return new Promise((resolve, reject) => {
            const callback = (err, decoded) => {
                if (err)
                    return reject(err);
                else
                    return resolve(decoded);
            };
            if (options)
                jwt.verify(token, secret, options, callback);
            else
                jwt.verify(token, secret, callback);
        });
    }
    /**
     * Creates token response and triggers callback
     * @param data {payload: object, options: object}
     * @param config: {config: IConfigValues}
     */
    static async tokenResponse(data, config) {
        if (!config.redirect_client_uri || !config.token_secret) {
            throw new Error(`Undefined: recirect_client_uri ${config.redirect_client_uri} or token_secret ${config.token_secret}`);
        }
        const { payload, options } = data.authorizationToken;
        try {
            const authToken = await this.createToken(payload, config.token_secret, options);
            const params = Object.assign(Object.assign({}, data), { authorizationToken: authToken });
            return { url: this.urlBuilder(config.redirect_client_uri, params) };
        }
        catch (error) {
            throw new Error(`tokenResponse error: ${error}`);
        }
    }
    /**
     *
     * Creates error response and triggers callback
     * @param params
     * @param config {config: IConfigValues}
     */
    static errorResponse(params, config) {
        if (!config.redirect_client_uri) {
            throw new Error('Missing redirect_client_uri');
        }
        return { url: this.urlBuilder(config.redirect_client_uri, params) };
    }
    /**
     * Generates Policy for AWS Api Gateway custom authorize
     * @param principalId {string} data for principalId field
     * @param effect {string} 'Allow' or 'Deny'
     * @param resource {string} method arn e.g. event.methodArn
     *  (arn:aws:execute-api:<regionId>:<accountId>:<apiId>/<stage>/<method>/<resourcePath>)
     */
    static generatePolicy(principalId, effect, resource) {
        if (!effect || !resource)
            throw 'Missing effect or resource';
        const statementOne = {
            Action: 'execute-api:Invoke',
            Effect: effect,
            Resource: resource
        };
        const policyDocument = {
            Version: '2012-10-17',
            Statement: [statementOne]
        };
        const authResponse = {
            principalId: principalId,
            policyDocument: policyDocument
        };
        return authResponse;
    }
    static hasKeys(item) {
        for (const key in item) {
            if (key)
                return true;
        }
        return false;
    }
}
exports.Utils = Utils;
/**
 * Simple way to check for undefined value and throw error for multiple fields.
*/
Utils.throwUndefined = (properties, message) => {
    const found = Object.entries(properties).filter(v => !v[1]).map(v => v[0]);
    const errorMessage = [message, `These fields cannot be undefined: ${found.join(', ')}`].filter(v => !!v);
    if (found.length !== 0)
        throw new Error(errorMessage.join('\n'));
};
